/* An addition Copyright (C) 2004 by Gerd Stolpmann */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif

#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include "filesize.h"

/* We assume here that "long" is a 32 bit signed integer (or better) */

void FILESIZE_Init (FILESIZE_CTX *ctx) {
  ctx->size = 0;
}

void FILESIZE_Update (FILESIZE_CTX *ctx, const unsigned char *buf, int bufsize) {
  long m;

  m = 0x7fffffff;
  if (ctx->size < 0 || m - bufsize < ctx->size) {
    ctx->size = -1;
  } else {
    ctx->size += bufsize;
  }
}


char * FILESIZE_End (FILESIZE_CTX *ctx, char *buf) {
  if (ctx->size < 0) return "ERROR";
  if (buf == NULL) buf = malloc(10);
  if (buf == NULL) return "ERROR";
  sprintf(buf, "%d", ctx->size);
  return buf;
}


char * FILESIZE_File (char *filename, char *buf) {
  struct stat sb;
  long size;
  long m;

  m = 0x7fffffff;
  if (stat(filename, &sb) < 0) return NULL;
  if (sb.st_size > m) return NULL;
  size = sb.st_size;

  if (buf == NULL) buf = malloc(10);
  if (buf == NULL) return NULL;
  sprintf(buf, "%d", size);
  return buf;
}
