/*	$NetBSD: pen.c,v 1.12 2003/04/11 21:25:58 grant Exp $	*/

#if 0
#include <sys/cdefs.h>
#ifndef lint
#if 0
static const char *rcsid = "from FreeBSD Id: pen.c,v 1.25 1997/10/08 07:48:12 charnier Exp";
#else
__RCSID("$NetBSD: pen.c,v 1.12 2003/04/11 21:25:58 grant Exp $");
#endif
#endif
#endif

/*
 * FreeBSD install - a package for the installation and maintainance
 * of non-core utilities.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * Jordan K. Hubbard
 * 18 July 1993
 *
 * Routines for managing the "play pen".
 *
 */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_ERR_H
#include <err.h>
#endif

#include "lib.h"

#ifdef HAVE_SIGNAL_H
#include <signal.h>
#endif

#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif

#ifdef HAVE_SYS_MOUNT_H
#include <sys/mount.h>
#endif

#ifdef HAVE_SYS_VFS_H
#include <sys/vfs.h>
#endif

/* For keeping track of where we are */
static char Current[GODI_FILENAME_MAX];
static char Previous[GODI_FILENAME_MAX];
static int CurrentSet;		/* rm -rf Current only if it's really set! */
                                /* CurrentSet is set to 0 before strcpy()s
				 * to prevent rm'ing of a partial string
				 * when interrupted by ^C */

#if 0
/*
 * Backup Current and Previous into temp. strings that are later
 * restored & freed by restore_dirs
 * This is to make nested calls to make_playpen/leave_playpen work
 */
void
save_dirs(char **c, char **p)
{
	*c = strdup(Current);	/* XXX */
	*p = strdup(Previous);
}

/*
 * Restore Current and Previous from temp strings that were created
 * by safe_dirs.
 * This is to make nested calls to make_playpen/leave_playpen work
 */
void
restore_dirs(char *c, char *p)
{
	CurrentSet = 0;		/* prevent from deleting */
	strcpy(Current, c);
	CurrentSet = 1;		/* rm -fr Current is safe now */
	free(c);
	
	strcpy(Previous, p);
	free(p);
}
#endif

char   *
where_playpen(void)
{
	return Current;
}

/*
 * Find a good place to play.
 */
static char *
find_play_pen(char *pen, size_t pensize, size_t sz)
{
	char   *cp;
	struct stat sb;

	if (pen && pen[0] && stat(pen, &sb) != FAIL)
		return pen;
	else if ((cp = getenv("GODI_TMPDIR")) != NULL && stat(cp, &sb) != FAIL)		        (void) snprintf(pen, pensize, "%s/instmp.XXXXXX", cp);
	else if ((cp = getenv("TMPDIR")) != NULL && stat(cp, &sb) != FAIL)
		(void) snprintf(pen, pensize, "%s/instmp.XXXXXX", cp);
	else if (stat("/var/tmp", &sb) != FAIL)
		strcpy(pen, "/var/tmp/instmp.XXXXXX");
	else if (stat("/tmp", &sb) != FAIL)
		strcpy(pen, "/tmp/instmp.XXXXXX");
	else if (stat("/usr/tmp", &sb) != FAIL)
		strcpy(pen, "/usr/tmp/instmp.XXXXXX");
	else {
		cleanup(0);
		errx(2,
		    "can't find temporary space to extract the files, please set your\n"
		    "GODI_TMPDIR environment variable to a location with at least %lu bytes\n"
		    "free", (u_long) sz);
		return NULL;
	}
	return pen;
}

#if !defined(HAVE_MKDTEMP)
static char *
mkdtemp(char *template)
{
	/* mkdtemp(3) is not present on 1.3.3 and below */
	if (!mktemp(template)) {
		cleanup(0);
		errx(2, "can't mktemp '%s'", template);
	}
	if (mkdir(template, 0700) < 0) {
		cleanup(0);
		errx(2, "can't mkdir '%s'", template);
	}
	return template;
}
#endif

/*
 * Make a temporary directory to play in and chdir() to it, returning
 * pathname of previous working directory.
 */
char   *
make_playpen(char *pen, size_t pensize, size_t sz)
{
	if (!find_play_pen(pen, pensize, sz))
		return NULL;

	if (!mkdtemp(pen)) {
		cleanup(0);
		errx(2, "can't mkdtemp '%s'", pen);
	}
	if (Verbose) {
		if (sz)
			fprintf(stderr,
		"Requested space: %lu bytes in %s\n",
			    (u_long) sz, pen);
	}
	if (Current[0])
		strcpy(Previous, Current);
	else if (!getcwd(Previous, GODI_FILENAME_MAX)) {
		cleanup(0);
		err(EXIT_FAILURE, "fatal error during execution: getcwd");
	}
	if (chdir(pen) == FAIL) {
		cleanup(0);
		errx(2, "can't chdir to '%s'", pen);
	}
	CurrentSet = 0; strcpy(Current, pen); CurrentSet = 1;
	
	return Previous;
}

/*
 * Convenience routine for getting out of playpen
 */
void
leave_playpen(char *save)
{
	void    (*oldsig) (int);

	/* Make us interruptable while we're cleaning up - just in case... */
	oldsig = signal(SIGINT, SIG_DFL);
	if (Previous[0] && chdir(Previous) == FAIL) {
		cleanup(0);
		errx(2, "can't chdir back to '%s'", Previous);
	} else if (CurrentSet && Current[0] && strcmp(Current, Previous)) {
		if (strcmp(Current, "/") == 0) {
			fprintf(stderr, "PANIC: About to rm -rf / (not doing so, aborting)\n");
			abort();
		}
		if (vsystem("%s -rf '%s'", RM, Current))
			warnx("couldn't remove temporary dir '%s'", Current);
		strcpy(Current, Previous);
	}
	if (save)
		strcpy(Previous, save);
	else
		Previous[0] = '\0';
	signal(SIGINT, oldsig);
}
